unit DelphiHelperUnit;

interface

uses Windows, RoutixRPCCOM_TLB, SysUtils;

type
  TOnAuthenticationEvent =
    function (const AUserName, APassword: WideString): WordBool of object;
  TOnRequestObjectEvent =
    function (const AObjectIDString: WideString): OleVariant of object;

function CreateRoutixRPCServer(AOnAuthenticationEvent: TOnAuthenticationEvent;
  AOnRequestObjectEvent: TOnRequestObjectEvent): IServer;

function CreateRoutixRPCClient: IClient;

implementation

type
  TRPCEventsHelper = class(TInterfacedObject, IServerEventsHelper)
  private
    FOnAuthenticationEvent: TOnAuthenticationEvent;
    FOnRequestObjectEvent: TOnRequestObjectEvent;
    function OnAuthentication(const AUserName: WideString;
      const APassword: WideString): WordBool; stdcall;
    function OnRequestObject(
      const AObjectIDString: WideString): OleVariant; stdcall;
  public
    constructor Create(AOnAuthenticationEvent: TOnAuthenticationEvent;
      AOnRequestObjectEvent: TOnRequestObjectEvent);
  end;

function CreateRoutixRPCServer(AOnAuthenticationEvent: TOnAuthenticationEvent;
  AOnRequestObjectEvent: TOnRequestObjectEvent): IServer;
begin
  Result := CoServer.Create;
  Result.SetEventsHelper(TRPCEventsHelper.Create(AOnAuthenticationEvent,
    AOnRequestObjectEvent) as IServerEventsHelper);
end;

function CreateRoutixRPCClient: IClient;
begin
  Result := CoClient.Create;
end;

{ TRPCEventsHelper }

constructor TRPCEventsHelper.Create(
  AOnAuthenticationEvent: TOnAuthenticationEvent;
  AOnRequestObjectEvent: TOnRequestObjectEvent);
begin
  inherited Create;
  FOnAuthenticationEvent := AOnAuthenticationEvent;
  FOnRequestObjectEvent := AOnRequestObjectEvent;
  if (not Assigned(FOnAuthenticationEvent)) or (not Assigned(FOnRequestObjectEvent)) then
    raise Exception.Create('One or more server events not assigned!');
end;

function TRPCEventsHelper.OnAuthentication(const AUserName: WideString;
  const APassword: WideString): WordBool;
begin
  Result := FOnAuthenticationEvent(AUserName, APassword);
end;

function TRPCEventsHelper.OnRequestObject(
  const AObjectIDString: WideString): OleVariant;
begin
  Result := FOnRequestObjectEvent(AObjectIDString);
end;

end.
